/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.similar.results.web.internal.contributor.asset.publisher;

import com.liferay.asset.kernel.model.AssetEntry;
import com.liferay.asset.kernel.model.AssetRenderer;
import com.liferay.asset.kernel.service.AssetEntryLocalService;
import com.liferay.asset.publisher.constants.AssetPublisherPortletKeys;
import com.liferay.journal.model.JournalArticle;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.search.similar.results.web.internal.builder.AssetTypeUtil;
import com.liferay.portal.search.similar.results.web.internal.util.SearchStringUtil;
import com.liferay.portal.search.similar.results.web.internal.util.http.HttpHelper;
import com.liferay.portal.search.similar.results.web.spi.contributor.SimilarResultsContributor;
import com.liferay.portal.search.similar.results.web.spi.contributor.helper.CriteriaBuilder;
import com.liferay.portal.search.similar.results.web.spi.contributor.helper.CriteriaHelper;
import com.liferay.portal.search.similar.results.web.spi.contributor.helper.DestinationBuilder;
import com.liferay.portal.search.similar.results.web.spi.contributor.helper.DestinationHelper;
import com.liferay.portal.search.similar.results.web.spi.contributor.helper.RouteBuilder;
import com.liferay.portal.search.similar.results.web.spi.contributor.helper.RouteHelper;

import java.util.Objects;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;

/**
 * @author Wade Cao
 * @author André de Oliveira
 */
@Component(service = SimilarResultsContributor.class)
public class AssetPublisherSimilarResultsContributor
	implements SimilarResultsContributor {

	@Override
	public void detectRoute(
		RouteBuilder routeBuilder, RouteHelper routeHelper) {

		String urlString = routeHelper.getURLString();

		String[] parameters = _httpHelper.getFriendlyURLParameters(urlString);

		SearchStringUtil.requireEquals("asset_publisher", parameters[0]);

		putAttribute(parameters[2], "type", routeBuilder);

		String assetEntryId = _httpHelper.getPortletIdParameter(
			urlString, "assetEntryId",
			_getAssetPublisherPortletId(parameters[1]));

		putAttribute(Long.valueOf(assetEntryId), "entryId", routeBuilder);
	}

	@Override
	public void resolveCriteria(
		CriteriaBuilder criteriaBuilder, CriteriaHelper criteriaHelper) {

		Long entryId = (Long)criteriaHelper.getRouteParameter("entryId");

		AssetEntry assetEntry = _assetEntryLocalService.fetchAssetEntry(
			entryId);

		if (assetEntry == null) {
			return;
		}

		criteriaBuilder.type(
			assetEntry.getClassName()
		).uid(
			_getUID(assetEntry)
		);
	}

	@Reference(unbind = "-")
	public void setAssetEntryLocalService(
		AssetEntryLocalService assetEntryLocalService) {

		_assetEntryLocalService = assetEntryLocalService;
	}

	@Reference(unbind = "-")
	public void setHttpHelper(HttpHelper httpHelper) {
		_httpHelper = httpHelper;
	}

	@Override
	public void writeDestination(
		DestinationBuilder destinationBuilder,
		DestinationHelper destinationHelper) {

		Long entryId = (Long)destinationHelper.getRouteParameter("entryId");

		String type = (String)destinationHelper.getRouteParameter("type");

		AssetEntry assetEntry = destinationHelper.getAssetEntry();

		destinationBuilder.replace(
			type,
			AssetTypeUtil.getAssetTypeByClassName(
				destinationHelper.getClassName())
		).replace(
			String.valueOf(entryId), String.valueOf(assetEntry.getEntryId())
		);
	}

	protected void putAttribute(
		Object value, String name, RouteBuilder routeBuilder) {

		routeBuilder.addAttribute(name, value);
	}

	private String _getAssetPublisherPortletId(String instanceId) {
		return AssetPublisherPortletKeys.ASSET_PUBLISHER + "_INSTANCE_" +
			instanceId;
	}

	private Long _getId(AssetEntry assetEntry) {
		if (Objects.equals(
				JournalArticle.class.getName(), assetEntry.getClassName())) {

			AssetRenderer<?> assetRenderer = assetEntry.getAssetRenderer();

			JournalArticle journalArticle =
				(JournalArticle)assetRenderer.getAssetObject();

			if (journalArticle == null) {
				return null;
			}

			return journalArticle.getId();
		}

		return assetEntry.getClassPK();
	}

	private String _getUID(AssetEntry assetEntry) {
		Long id = _getId(assetEntry);

		if (id == null) {
			return null;
		}

		return Field.getUID(assetEntry.getClassName(), String.valueOf(id));
	}

	private AssetEntryLocalService _assetEntryLocalService;
	private HttpHelper _httpHelper;

}